﻿using System;
using System.Net;
using System.Windows;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections;

namespace AsycValidation
{
    public class BasicViewModel : INotifyPropertyChanged, INotifyDataErrorInfo
    {
        public event PropertyChangedEventHandler PropertyChanged;
        public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;


        private Dictionary<string, List<ValidationErrorInfo>> _errors =
                      new Dictionary<string, List<ValidationErrorInfo>>();


        protected void RemoveErrorFromPropertyAndNotifyErrorChanges(
                                 string propertyName,
                                 int errorCode)
        {
            if (_errors.ContainsKey(propertyName))
            {
                RemoveErrorFromPropertyIfErrorCodeAlreadyExist(propertyName, errorCode);

                NotifyErrorsChanged(propertyName);
            }
        }

        private void RemoveErrorFromPropertyIfErrorCodeAlreadyExist(
                                 string propertyName,
                                 int errorCode)
        {
            if (_errors.ContainsKey(propertyName))
            {
                var errorToRemove = _errors[propertyName].SingleOrDefault(
                                       error => error.ErrorCode == errorCode);

                if (errorToRemove != null)
                {
                    _errors[propertyName].Remove(errorToRemove);




                    if (_errors[propertyName].Count == 0)
                        _errors.Remove(propertyName);
                }
            }
        }
        protected void AddErrorToPropertyAndNotifyErrorChanges(
                                  string propertyName,
                                  ValidationErrorInfo errorInfo)
        {
            RemoveErrorFromPropertyIfErrorCodeAlreadyExist(propertyName, errorInfo.ErrorCode);
            if (!_errors.ContainsKey(propertyName))
                _errors.Add(propertyName, new List<ValidationErrorInfo>());

            _errors[propertyName].Add(errorInfo);

            NotifyErrorsChanged(propertyName);
        }


        public IEnumerable GetErrors(string propertyName)
        {
            if (!_errors.ContainsKey(propertyName))
                return _errors.Values;

            return _errors[propertyName];
        }


        public bool HasErrors
        {
            get { return this._errors.Count > 0; }
        }


        private void NotifyErrorsChanged(string propertyName)
        {
            if (ErrorsChanged != null)
                ErrorsChanged(this, new DataErrorsChangedEventArgs(propertyName));
        }


        protected void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

    }
}
